


extern "C" {
   #include <psim.h>

   void paircntr_(void);
}

//======================================================
// Here are some parameters defining the geometry of the
// e+/e- pair counting luminosity monitor.
//
// The size of the downsteam detector is determined by
// the size of the upstream detector and their relative
// distance such that they cover the same solid angle.
//
// The geometry is designed assuming all particles are
// comming from a point on the beamline at Z_PAIR_ORIGIN
// cm. downstream from the target.
//
// The center of the front face (FFC) of the upstream
// detector will be positioned at a z coordinate that is 
// UPSTREAM_PAIRCNTR_Z_POS cm downstream of the target.
//
// The FFC of the downstream detector will be postitioned
// as a z coordinate that is 
// UPSTREAM_DOWNSTREAM_SEPARATION cm away from the FFC
// of the upstream detector. This distance is measured
// along the line connecting the upstream FFC to the
// Z_PAIR_ORIGIN point on the beamline.
//
// The FFC of the upstream detector will be positioned
// at an angle of UPSTREAM_PAIRCNTR_THETA_START degrees
// from the beamline. The rotation is done in the x-z
// plane about the axis passing through Z_PAIR_ORIGIN
// on the beamline.
//=======================================================

// These are all in cm.
float Z_PAIR_ORIGIN=10.0;
float UPSTREAM_PAIRCNTR_HEIGHT=15.0;
float UPSTREAM_PAIRCNTR_WIDTH =5.0;
float UPSTREAM_PAIRCNTR_DEPTH =0.5;
float UPSTREAM_PAIRCNTR_Z_POS =350.0;
float UPSTREAM_DOWNSTREAM_SEPARATION=100.0;

// These are all in degrees
float UPSTREAM_PAIRCNTR_THETA_START=4.2;



void paircntr_(void)
{
   float PAR[3];
   int IVOL,ISET,IDET;
   float theta,del_theta,rot_theta;
   float r,x,y,z,d;
   int i;
   float th1,ph1,th2,ph2,th3,ph3;
   
   // define upstream detector shape
   PAR[0]=UPSTREAM_PAIRCNTR_WIDTH/2.0;
   PAR[1]=UPSTREAM_PAIRCNTR_HEIGHT/2.0;
   PAR[2]=UPSTREAM_PAIRCNTR_DEPTH/2.0;
   gsvolu("PCUP","BOX ", 5 ,PAR,3,&IVOL);

   // define downstream detector shape
   z=UPSTREAM_PAIRCNTR_Z_POS-Z_PAIR_ORIGIN;
   d=UPSTREAM_DOWNSTREAM_SEPARATION;
   PAR[0] *= (z+d)/z;
   PAR[1] *= (z+d)/z;
   gsvolu("PCDN","BOX ", 5 ,PAR,3,&IVOL);
   
   
   // position detectors
   theta=DEG2RAD*UPSTREAM_PAIRCNTR_THETA_START;
   r=(UPSTREAM_PAIRCNTR_Z_POS-Z_PAIR_ORIGIN)/cos(theta);
   r+=0.5*UPSTREAM_PAIRCNTR_DEPTH;
   del_theta=2.0*atan(0.5*UPSTREAM_PAIRCNTR_WIDTH/r);
   y=0;
   for(i=1;i<=8;i++){

      // rotation matrix right side
      rot_theta=(theta/DEG2RAD);
      th1=90.0-rot_theta;
      ph1=0.0;
      th2=90.0;
      ph2=90.0;
      th3=rot_theta;
      ph3=180.0;
      gsrotm(100+i,th1,ph1,th2,ph2,th3,ph3);

      // rotation matrix left side
      th1=90.0+rot_theta;
      ph1=0.0;
      th2=90.0;
      ph2=90.0;
      th3=rot_theta;
      ph3=0.0;
      gsrotm(110+i,th1,ph1,th2,ph2,th3,ph3);
      
      // left front
      z=(r*cos(theta)) + Z_PAIR_ORIGIN;
      x=r*sin(theta);
      gspos("PCUP" ,2000+i, "HALB" ,x,y,z, 110+i,"ONLY");

      // right front
      x=-x;
      gspos("PCUP" ,2008+i, "HALB" ,x,y,z, 100+i,"ONLY");
      
      // left back
      z=((r+UPSTREAM_DOWNSTREAM_SEPARATION)*cos(theta)) + Z_PAIR_ORIGIN;
      x=(r+UPSTREAM_DOWNSTREAM_SEPARATION)*sin(theta);
      gspos("PCDN" ,2016+i, "HALB" ,x,y,z, 110+i,"ONLY");

      // right back
      x=-x;
      gspos("PCDN" ,2024+i, "HALB" ,x,y,z, 100+i,"ONLY");

      theta+=del_theta;
   }
   
   // Add both PCUP and PCDN to detector set PAIR
   gsdetv("PAIR","PCUP",3,100,100,&ISET,&IDET);
   gsdetv("PAIR","PCDN",3,100,100,&ISET,&IDET);
}




